/*
 * example.cc - An example program illustrating how to use the Evolutionary
 * Algorithms Library.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */


#include "ExperimentC.h"
#include "ConfigurationC.h"
#include "StatisticsC.h"
#include "Common/RandomC.h"

#include "Core/CoreFactoryC.h"
#include "MyProblemFactoryC.h"
#include "Reproduction/ReproductionFactoryC.h"

#include "Error/ErrorC.h"
#include "Error/WarningC.h"
#include "Error/InternalErrorC.h"
#include "Error/LibraryErrorC.h"

int main()
{
	try
	{

		/*
		 *  By using a configuration file, we can control all kind of parameters
		 *  when the program is compiled.
		 */

		cerr << "Opening configuration file\n  [configuration]";

		ConfigurationA * configuration;
		configuration = new ConfigurationC("example.ini");


		/*
		 * We need a statistics class for collecting information.
		 *
		 */

		cerr << "\nCreating statistics\n  [statistics]";

		StatisticsA * statistics;
		statistics = new StatisticsC(*configuration);


		/*
		 *  The library needs a randomgenerator, which we will have to provide,
		 *  luckily the library also provides a simple generator.
		 */

		cerr << "\nCreating randomgenerator\n  [randomgenerator]";

		RandomA * randomgenerator;
		randomgenerator = new RandomC();


		/*
		 *  First things first, one factory has to be made for each
		 *  package.
		 */

		cerr << "\nCreating factories";

		cerr << "\n  [corefactory]";
		CoreFactoryA * corefactory;
		corefactory = new CoreFactoryC(*configuration, *randomgenerator);

		cerr << "\n  [problemfactory]";
		ProblemFactoryA * problemfactory;
		problemfactory = new MyProblemFactoryC(*configuration);

		cerr << "\n  [reproductionfactory]";
		ReproductionFactoryA * reproductionfactory;
		reproductionfactory = new ReproductionFactoryC(*configuration, *randomgenerator);

		/*
		 *  We let the Experiment class produce the problem and evolutionary
		 *  algorithm for us.
		 */

		cerr << "\nCreating experiment manager\n  [experiment]";

		ExperimentA * experiment;
		experiment = new ExperimentC(*configuration, *statistics, *corefactory, *problemfactory, *reproductionfactory, *randomgenerator);


		/*
		 *  All setup and ready to go, it is now time to run the experiment.
		 */

		cerr << "\nRunning experiment\n";

		experiment->Run ();


		/*
		 *  EA is done, time to shut down the program, but first we have
		 *  to clean up our memory.
		 */

		cerr << "\nFreeing memory";

		cerr << "\n  [experiment]";
		delete experiment;
		cerr << "\n  [problemfactory]";
		delete problemfactory;
		cerr << "\n  [corefactory]";
		delete corefactory;
		cerr << "\n  [reproductionfactory]";
		delete reproductionfactory;
		cerr << "\n  [randomgenerator]";
		delete randomgenerator;
		cerr << "\n  [statistics]";
		delete statistics;
		cerr << "\n  [configuration]";
		delete configuration;

	}
	catch(ExceptionA * Exception)
	{
		Exception->Handle();
		delete Exception;
	}
	catch(...)
	{
		cerr << "Signal 11. Someone has thrown an exception, but it is not clear who and of what kind it is.\n";
	}

	cerr << "\n";
	return (0);
}
