/*
 * ArrayC.h - Template class implementing an array with buildin checks.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 09-Jun-1998
 *
 */

#ifndef ArrayC_h
#define ArrayC_h 1

#include "Error/InternalErrorC.h"
#include "Common/StringC.h"


/** This template array-class has the advantage of checking numerous things, and giving sensible errors when something is going wrong. Furthermore it supports the naming of arrays, so you can have a unique name for every array. When an error occurs, you know instantly which array is the troublemaker. As most of the member functions behaviour is obvious, only some of them have additional explanation.
 *
 * @memo A general array-class
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

template <class T>
class ArrayC
{
	public:

		/** 
		 * @name Constructors
		 */

		//@{

		/** This is the wo/name constructor. When using this constructor errors will be reported without a name.
		 *
		 * @memo Creating a new array
		 *
		 * @param iSize Size of the array.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		ArrayC
		(
			unsigned int iSize
		)
		{
			Name = 0;
			Init (iSize);
		}


		/** This is the w/name constructor. When using this constructor errors will be reported witha name that identifies the array.
		 *
		 * @memo Creating a new array
		 *
		 * @param iSize Size of the array.
		 * @param iName A useful description of the array
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		ArrayC
		(
			unsigned int iSize,
			StringC const & iString
		)
		{
			Name = new StringC(iString);
			Init (iSize);
		}


		/** Copy constructor, destroys this array and copies the given one into it.
		 *
		 * @memo Copying an array
		 *
		 * @param iArray Another array of this class.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		ArrayC
		(
			ArrayC<T> const & iArray
		);

		//@}

		~ArrayC ()
		{
			delete [] Data;
			if (Name != 0)
				delete Name;
		}

		ArrayC<T> const & operator=
		(
			ArrayC<T> const & iArray
		);

		bool const operator==
		(
			ArrayC<T> const & iArray
		);

		unsigned int GetSize () const;

		T & operator[]
		(
			unsigned int iIndex
		) throw(ExceptionA*);

		T operator[]
		(
			unsigned int iIndex
		) const throw(ExceptionA*);

	protected:

	private:

		unsigned int Size;

		T * Data;

		void const Init
		(
			unsigned int const iSize
		) throw(ExceptionA*);

		StringC const * Name;

}; // ArrayC

#endif

// Eof ArrayC.h
