/*
 * RandomA.h - Abstract class for a random-generator.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */


#ifndef RandomA_h
#define RandomA_h

#include "typedef.h"


/** This interface for a random-generator is included because of the fixed datatype used in the library. It has the basic functions of a random-generator, such as setting the seed and getting numbers within a range.
 *
 * @memo A randomgenerator
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class RandomA
{
	public:

		/** This function sets the seeds and then reseeds the generator.
		 *
		 * @memo Providing a seed
		 *
		 * @param iSeed The seed for the random-generator.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void SetSeed
		(
			DataType const & iSeed
		);


		virtual ~RandomA() =0;


		/** 
		 * \begin{description} \item[] Example:  \item[]{\tt void ShowRandomNumber(RandomA const #&# random)} \item[] #{# \begin{description} \item[]{\tt unsigned int r = random;} \item[]{\tt cout << r << endl;}  \end{description} \item[] #}# \end{description}
		 * This function gets a random number. The range of this number depends on the datatype defined in the library.
		 *
		 * @memo Getting a random number
		 *
		 * @return A random number.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual operator DataType () =0;


		/** 
		 * \begin{description} \item[] Example:  \item[]{\tt void ShowRandomNumberUnderTen(RandomA const #&# random)} \item[] #{# \begin{description} \item[]{\tt unsigned int r = random(10);} \item[]{\tt cout << r << endl;} \end{description} \item[] #}#  \end{description}
		 * This function gets a random number within the range zero to the number provided.
		 *
		 * @memo Getting a random number in a range
		 *
		 * @param iRange Size of range where the random number should be from, range will be $0\ldots(\texttt{iRange}-1)$.
		 * @return Random number.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		 
		virtual DataType operator()
		(
			DataType iRange
		) const =0;

		void Reseed();

	protected:

		/** 
		 * Provided with a seed, this function should initialize the random-generator.
		 *
		 * @memo Initializing the random-generator
		 *
		 * @param iSeed Seed used for initializing the random-generator.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		 
		virtual void DoSeed
		(
			DataType const & iSeed
		) =0;

		DataType Seed;

		RandomA()
		{
			Seed = 1;
		};

	private:

}; // RandomA

#endif
