/*
 * TreeC.h - A general n-ary tree.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Aug-1998
 *
 */

#ifndef TreeC_h
#define TreeC_h

#include <Error/LibraryErrorC.h>
#include <Error/InternalErrorC.h>

#include <vector>
#include <stack>
#include <iostream>


template <class T>
class TreeC
{

	public:
		class TreeElementC;
		class Insert_IteratorC;

		// -----------------------------  Iterators

		class IteratorA
		{
			public:

				virtual ~IteratorA(){};

				virtual bool IsEnd() const =0;

				virtual IteratorA & operator++() =0;
				virtual IteratorA & operator++
				(
					int iInt
				) =0;

				virtual T & operator*() const =0;
				virtual T * operator->() const =0;

				virtual bool IsLeaf()
				{
					return (Current->Child == 0);
				};

				virtual TreeElementC const & GetTreeElement() const
				{
					 return *Current;
				};

				friend Insert_IteratorC;

			protected:

				IteratorA
				(
					TreeElementC * iTree
				): Current(iTree)
				{
				};

				TreeElementC * Current;

			private:

		}; // IteratorA

		class Insert_IteratorC
		{
			public:

				Insert_IteratorC
				(
				 	TreeC<T> & iTree,
					IteratorA & iIterator
				): Tree(iTree), Current(iIterator.Current)
				{};

				Insert_IteratorC
				(
					Insert_IteratorC const & iInsert_Iterator
				): Tree(iInsert_Iterator.Tree), Current(iInsert_Iterator.Current)
				{};

	 			virtual ~Insert_IteratorC(){};

				virtual T* operator->() const
				{
					return Current->Element;
				};

				virtual void AddChild
				(
					T & iElement
				);

			protected:

				TreeC<T> & Tree;

				TreeElementC * Current;

			private:

		}; // Insert_IteratorC


		class ConstIteratorA
		{
			public:

				ConstIteratorA
				(
					TreeElementC const & iTree
				): Tree(iTree)
				{
				};

				virtual ~ConstIteratorA() =0;

				virtual void First();
				virtual void Next();

			protected:

				TreeElementC const & Tree;

			private:

		}; // ConstIteratorA

		class ConstIterator_PreorderC: public ConstIteratorA
		{
			public:

				ConstIterator_PreorderC
				(
					TreeElementC const & iTree
				): ConstIteratorA(iTree),Current(iTree)
				{};

	 			virtual ~ConstIterator_PreorderC();

				virtual void First();
				virtual void Next();

			protected:
				
				TreeElementC const & Current;

			private:

		}; // Iterator_PreorderC


		class Iterator_PreorderC: public IteratorA
		{
			public:

				Iterator_PreorderC
				(
					TreeElementC * iTree
				): IteratorA(iTree)
				{};

	 			virtual ~Iterator_PreorderC(){};

				virtual IteratorA & operator++();
				virtual IteratorA & operator++(int iInt)
				{
					if (iInt == 0)
						iInt = 1;
					for (int i = 0; i < iInt; ++i)
					{
						 ++(*this);
					}
					return *this;
				};

				virtual T* operator->() const
				{
					return &(Current->Element);
				};

				virtual T & operator*() const
				{
					return Current->Element;
				};

			protected:

				stack<TreeElementC *> StillToDo;

			private:

		}; // Iterator_PreorderC

		class Iterator_PostorderC: public IteratorA
		{
			public:

	 			virtual ~Iterator_PostorderC(){};

				virtual bool IsEnd() const
				{
					 return (Current == 0);
				};

				virtual IteratorA & operator++();
				virtual IteratorA & operator++(int iInt)
				{
					if (iInt == 0)
						iInt = 1;
					for (int i = 0; i < iInt; ++i)
					{
						 ++(*this);
						 if (this->Current == 0)
							 return 0;
					}

					return *this;
				};

				virtual T* operator->() const
				{
					if (Current != 0)
						return Current->Element;
					else
						throw new InternalErrorC("Tried to read past the end of a tree.");
				};

				virtual T & operator*() const
				{
					if (Current != 0)
						return *(Current->Element);
					else
						throw new InternalErrorC("Tried to read past the end of a tree.");
				};

				friend TreeC<T>;

			protected:

				stack<TreeElementC *> StillToDo;

				Iterator_PostorderC
				(
					TreeElementC * iTree
				): IteratorA(iTree)
				{
					if (Current != 0)
					{
						while (Current->Child != 0)
						{
							StillToDo.push(Current);
							Current = Current->Child;
						}
						 
					}
				};

			private:

		}; // Iterator_PostorderC



		// ----------------------------------------  TreeElementC
		
		class TreeElementC
		{
			public:

				TreeElementC
				(
					T * iElement
				): Element(iElement)
				{
					Parent = 0;
					Child = 0;
					Left = 0;
					Right = 0;
				};
				
				TreeElementC
				(
					TreeElementC * iParent,
					T * iElement
				): Parent(iParent), Element(iElement)
				{
					Child = 0;
					if (Parent != 0)
					{
						Right = 0;
						if (Parent->Child == 0)
						{
							 Parent->Child = this;
							 Left = 0;
						}
						else
						{
							 TreeElementC * tmp = Parent->Child;
							 while (tmp->Right != 0)
								 tmp = tmp->Right;
							 tmp->Right = this;
							 Left = tmp;
								 
						}
							 
					}
					else
					{
						Left = 0;
						Right = 0;
					}
				};
				
				virtual ~TreeElementC()
				{
					cerr << "T";

					// No children, kill yourself and tie neighbors to each other, or if first child, make right neighbor new first child.
					/*
					if (Child == 0)
					{
						if (Left != 0)
						{
							Left->Right = Right;
							if (Right != 0)
							{
								Right->Left = Left;
							}
						}
						else
						{
							if (Parent != 0)
							{
								 Parent->Child = Right;
								 if (Right != 0)
								 {
									  Right->Left = 0;
								 }
							}
						}
					}
					else
					{
						if (Left != 0)
						{
							Left->Right = Child;
							TreeElementC * tmp = Child;
							tmp->Parent = Parent;
							while (tmp->Right != 0)
							{
							    tmp = tmp->Right;
							    tmp->Parent = Parent;
							}
							if (Right != 0)
							{
							     Right->Left = tmp;
							}
						}
						else
						{
							if (Parent != 0)
							{
								Parent->Child = Child;
							}
							TreeElementC * tmp = Child;
							tmp->Parent = Parent;
							while (tmp->Right != 0)
							{
							    tmp = tmp->Right;
							    tmp->Parent = Parent;
							}
							if (Right != 0)
							{
							     Right->Left = tmp;
							}
						}
					}
				*/

					//delete &Element;
				};


				friend class ChildConstIteratorC
				{
					public:

						virtual ~ChildConstIteratorC(){};

						virtual bool IsEnd() const
						{
							 return (Current == 0);
						};

						virtual ChildConstIteratorC & operator++()
						{
							if (Current == 0)
							{
								throw new InternalErrorC("Tried to walk past the end of a list of children.");
							}
	
							Current = Current->Right;

							return (*this);
						};
						
						virtual ChildConstIteratorC & operator++(int iInt)
						{
							if (iInt == 0)
								iInt = 1;
							for (int i = 0; i < iInt; ++i)
							{
								 ++(*this);
							}

							return *this;
						};

						virtual T* operator->() const
						{
							if (Current != 0)
								return Current->Element;
							else
								throw new InternalErrorC("Tried to read past the end of a list of children.");
						};

						virtual T & operator*() const
						{
							if (Current != 0)
								return *(Current->Element);
							else
								throw new InternalErrorC("Tried to read past the end of a list of children.");
						};

						ChildConstIteratorC
						(
							TreeElementC const & iTreeElement
						): TreeElement(iTreeElement)
						{
							Current = TreeElement.Child;
						};

						virtual TreeElementC const & GetTreeElement() const
						{
							 return *Current;
						};

					protected:

						TreeElementC const & TreeElement;

						TreeElementC * Current;

					private:
				};
				
				virtual ChildConstIteratorC CreateChildConstIterator() const
				{
					 return ChildConstIteratorC(*this);
				};

				friend TreeC<T>::Iterator_PreorderC;

				friend TreeC<T>::Iterator_PostorderC;

				friend TreeC<T>::Insert_IteratorC;

				friend TreeC<T>::IteratorA;

				// For Tree<T>::Print
				friend TreeC<T>;

				virtual T const & GetElement() const
				{
					 return *Element;
				};

			//protected:
				/// This container element's left and right brother
				TreeElementC * Right;
				TreeElementC * Left;

				/// Its parent and first child
				TreeElementC * Parent;
				TreeElementC * Child;

				T * Element;

			protected:

			private:

		}; // TreeElementC

		TreeC(): Root(0)
		{};

		virtual ~TreeC()
		{
			DeleteElement(Root);

			/*Iterator_PostorderC i = CreateIterator_Postorder();

			while (!i.IsEnd())
			{
				//TODO: Next line is not correct, but what should be used here?
				//i.GetTreeElement().~TreeElementC();
				delete &(i.GetTreeElement());
				++i;
			}*/
			
		};

		virtual TreeElementC const & GetRoot() const
		{
			 return *Root;
		};

		Iterator_PreorderC CreateIterator_Preorder()
		{
			 return Iterator_PreorderC(Root);
		};

		Iterator_PostorderC CreateIterator_Postorder()
		{
			 return Iterator_PostorderC(Root);
		};

		Insert_IteratorC CreateInsert_Iterator
		(
			IteratorA & iIterator
		)
		{
			 return Insert_IteratorC(*this, iIterator);
		};
		friend Insert_IteratorC;

		//TODO: Implement swap trees.
		friend void Swap
		(
		);

		friend ostream & operator<<
		(
			ostream & iOstream,
			TreeC<T> & const iTree
		)
		{
			iTree.Print(iOstream);

			return iOstream;
		};


	protected:

		void Print
		(
			ostream const & iOstream
		) const
		{
			if (Root != 0)
			{
				PrintElement(iOstream, *Root);
			}
			else
			{
				iOstream << "(null)";
			}
		};

		TreeElementC * Root;
	 
	private:

		void DeleteElement
		(
			TreeElementC * iElement
		)
		{
			TreeElementC * i = iElement->Child;
			while (i != 0)
			{
				DeleteElement(i);
				i = i->Right;
			}
			//cerr << " " << *(iElement->Element) << ";";
			delete iElement->Element;
			delete iElement;
		};

		void PrintElement
		(
			ostream const & iOstream,
			TreeElementC const & iElement
		)
		{
			bool children = false;
			TreeElementC * i = iElement.Child;
			if (i != 0)
			{
				iOstream << "(";
				children = true;
			}

			iOstream << *(iElement.Element);

			while (i != 0)
			{
				iOstream << " ";
				PrintElement(iOstream, *i);
				i = i->Right;
			}

			if (children)
			{
				iOstream << ")";
			}
		};
		
}; // TreeC

#endif

// Eof TreeElementC.h
