/*
 * CoreFactoryC.h - Abstract base class for instantiating objects within the Core package.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef CoreFactoryC_h
#define CoreFactoryC_h 1

#include "Core/CoreFactoryA.h"

#include "Core/EvolutionaryAlgorithmC.h"
#include "Core/PopulationC.h"
#include "Core/GenotypeC.h"
#include "Core/StopConditionC.h"
#include "Core/InitializationC.h"
#include "Core/PoolC.h"
#include "Core/PoolIteratorC.h"


/** This is the default factory for the Core package. When it is used in a program, all the simple examples in the library will be created.
 *
 * @memo Default factory of the Core package
 *
 * @see CoreFactoryA
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class CoreFactoryC: public CoreFactoryA
{
	public:


		/**
		 * \begin{description}
		 * \item[]{\tt CoreFactoryA * Corefactory}
		 * \begin{description} \item[]{\tt = new CoreFactoryC(Configuration);} \end{description}
		 * \end{description}
		 *
		 * @memo Constructing a new factory for the Core package
		 *
		 * @param iConfiguration A reference copy of configuration will be made and it will be distributed to every object made by the factory.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 * @see CoreFactoryA
		 */

		CoreFactoryC
		(
			ConfigurationA const & iConfiguration,
			RandomA const & iRandom
		): CoreFactoryA(iConfiguration, iRandom)
		{};


		virtual ~CoreFactoryC()
		{};


		/** See the abstract base class for further information.
		 *
		 * @name Create functions
		 *
		 * @memo Creating default objects in the Core package
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 * @see CoreFactoryA
		 *
		 */
		
		//@{
		///
		virtual EvolutionaryAlgorithmA* CreateEvolutionaryAlgorithm
		(
			StatisticsA & iStatistics,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			ProblemA const & iProblem
		) const;

		///
		virtual PopulationA* CreatePopulation
		(
			StatisticsA & iStatistics,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			StopConditionA const & iStopCondition,
			ProblemA const & iProblem
		) const;

		///
		virtual GenotypeA* CreateGenotype
		(
			ProblemFactoryA const & iProblemFactory
		) const;

		///
		virtual StopConditionA* CreateStopCondition() const;

		///
		virtual InitializationA* CreateInitialization
		(
			ProblemA const & iProblem
		) const;

		///
		virtual PoolA* CreatePool() const;
		//@}

	protected:

	private:

}; // CoreFactoryC

#endif

// Eof CoreFactoryC.h
