/*
 * EvolutionaryAlgorithmA.h - Abstract base class defining the interface of an evolutionary algorithm.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef EvolutionaryAlgorithmA_h
#define EvolutionaryAlgorithmA_h 1

class StopConditionA;
class PopulationA;
class StatisticsA;

class CoreFactoryA;
class ReproductionFactoryA;
class ProblemFactoryA;

class ConfigurationA;
class ProblemA;


/** This is the abstract class representing a basic Evolutionary Algorithm. It should run the loop that goes through all the generations.
 *
 * @memo Abstract Evolutionary Algorithm 
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class EvolutionaryAlgorithmA
{
	public:

		/** 
		 * This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters.
		 *
		 * @memo Constructor for the abstract EvolutionaryAlgorithm class.
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iStatistics The statistics are used to write data to and build a report after a run of the evolutionary algorithm.
		 * @param iCoreFactory The factory that is used to spawn objects from classes of the Core package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iReproductionFactory The factory that is used to spawn objects from classes of the Reproduction package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblem Actual problem the evolutionary algorithm has to work on.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		EvolutionaryAlgorithmA
		(
			ConfigurationA const & iConfiguration,
			StatisticsA & iStatistics,
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			ProblemA const & iProblem
		);


		virtual ~EvolutionaryAlgorithmA() = 0;


		/** Runs the evolutionary algorithm on the problem and writes statistics during and after the run.
		 *
		 * @memo Abstract method for running the evolutionary algorithm
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void Run() =0;

	protected:

		ConfigurationA const & Configuration;

		StatisticsA & Statistics;

		ProblemFactoryA const & ProblemFactory;
		CoreFactoryA const & CoreFactory;
		ReproductionFactoryA const & ReproductionFactory;

		ProblemA const & Problem;

	private:

}; // EvolutionaryAlgorithmA

#endif

// Eof EvolutionaryAlgorithmA.h
