/*
 * PopulationA.h - Abstract base class defining the interface of a population.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 23-Jun-1998
 *
 */

#ifndef PopulationA_h
#define PopulationA_h 1

class ConfigurationA;

class CoreFactoryA;
class ReproductionFactoryA;
class ProblemFactoryA;
class StatisticsA;

class StopConditionA;
class GenotypeA;
class ProblemA;


/** The population consists of a pool that holds the genomes and a reproduction unit that provides the population with a way to the next generation. In addtion to that it has a fitnessfunction for evaluating the genotypes in the pool.
 *
 * @memo The population
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class PopulationA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. Its children should create the following objects: \begin{itemize} \item[Initialization] which is used for initialising genotypes  \item[Reproduction] needed to let the population evolve to the next generation  \item[FitnessFunction] needed to evaluate genotypes  \item[Pool] an empty pool where the genotypes reside  \end{itemize}
		 *
		 * @memo Creating a new population
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iStatistics The statistics are used to write data to and build a report after a run of the evolutionary algorithm.
		 * @param iCoreFactory The factory that is used to spawn objects from classes of the Core package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iReproductionFactory The factory that is used to spawn objects from classes of the Reproduction package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblem Actual problem the evolutionary algorithm has to work on.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		PopulationA
		(
			ConfigurationA const & iConfiguration,
			StatisticsA & iStatistics,
		 	CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			StopConditionA const & iStopCondition,
			ProblemA const & iProblem
		);

		virtual ~PopulationA() =0;


		/** By calling this function the pool is filled with genotypes, these genotypes are initialised and evaluated before they are inserted into the pool.
		 *
		 * @memo Initializing the population
		 *
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual void Init() =0;


		/** This function should take the population to the next generation by using the Reproduction unit.
		 *
		 * @memo Evolving the population
		 *
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual void NextGeneration() =0;


		/** A function to get the best individual of the population.
		 *
		 * @memo Getting the best individual
		 *
		 * @return The best genotype according to the fittnessfunction.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		virtual GenotypeA const & GetBestIndividual() const =0;

	protected:

		unsigned int Size;

		ConfigurationA const & Configuration;

		StatisticsA & Statistics;

		CoreFactoryA const & CoreFactory;
		ProblemFactoryA const & ProblemFactory;
		ReproductionFactoryA const & ReproductionFactory;

		StopConditionA const & StopCondition;

		ProblemA const & Problem;

	private:

}; // PopulationA

#endif

// Eof PopulationA.h
