/*
 * ExperimentA.h - Abstract base class defining the interface of an experiment manager.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 18-Jun-1998
 *
 */

#ifndef ExperimentA_h
#define ExperimentA_h 1

#include "Core/EvolutionaryAlgorithmA.h"
#include "ConfigurationA.h"
#include "Core/CoreFactoryA.h"
#include "Problem/ProblemFactoryA.h"
#include "Reproduction/ReproductionFactoryA.h"
#include "Common/RandomA.h"
#include "StatisticsA.h"


#include <new.h>

/** The experiment will runs the appropriate tests on different parameter settings for the problem and/or evolutionary algorithm. After running a test it saves the gathered information contained in then statistics class to a file.
 *
 * @memo Managing experiments
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ExperimentA
{
	public:

		/**
		 * \begin{description}
		 * \item[] Example:
		 * \item[]{\tt ExperimentA * Experiment = new ExperimentC(Configuration, Statistics, CoreFactory, ProblemFactory, ReproductionFactory, Random);}
		 * \end{description}
		 *
		 * @memo Declaring a new Experiment
		 *
		 * @param iConfiguration The configuration is used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iStatistics The statistics are used to write data to and build a report after a run of the evolutionary algorithm.
		 * @param iCoreFactory The factory that is used to spawn objects from classes of the Core package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iReproductionFactory The factory that is used to spawn objects from classes of the Reproduction package, within this class. It is passed on to these spawned objects as well.
		 * @param iRandom This randomgenerator is given a seed and then passed on to underlying objects to serve as a source for random numbers.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		 
		ExperimentA
		(
			ConfigurationA const & iConfiguration,
			StatisticsA & iStatistics,
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			RandomA & iRandom
		);


		virtual ~ExperimentA() = 0;


		/**
		 * \begin{description}
		 * \item[] Example: 
		 * \item[]{\tt ExperimentA * Experiment = new ExperimentC(Configuration, Statistics, CoreFactory, ProblemFactory, ReproductionFactory, Random);}
		 * \begin{description} \item[] {\tt ...}  \end{description}
		 * \item[]{\tt Experiment->Run();}
		 * \end{description}
		 * This function should be implemented by the class inheriting from this one. It runs an experiment and collects statistical information. The experiment will consist of a problem and an evolutionary algorithm that tries to solve it. Here things as different parameter settings and multiple runs will be implemented.
		 *
		 * @memo Declaring a new Experiment
		 *
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		 
		virtual void Run() =0;

	protected:

		ConfigurationA const & Configuration;

		StatisticsA & Statistics;

		ProblemFactoryA const & ProblemFactory;
		CoreFactoryA const & CoreFactory;
		ReproductionFactoryA const & ReproductionFactory;

		RandomA & Random;

		/// Handler for when the program runs out of memory. For now it only prints a message and exits.
		static void OutOfMemory();

	private:

}; // ExperimentA

#endif

// Eof ExperimentA.h
