/*
 * DataA.h - Abstract base class defining the interface of data in the
 * genotype.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-Jun-1998
 *
 */

#ifndef DataA_h
#define DataA_h 1

#include "typedef.h"

#include <iostream.h>

/** This holds the data of a genotype. This data will be manipulated by the reproduction operators and will be evaluated by the fitnessfunction. It has iterators to hide the inner structure.
 *
 * @memo A genotype's data
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class DataA
{
	public:

		/** This iterator should be used when the data is to be viewed.
		 *
		 * @memo Viewing data
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		class DataConstIteratorA
		{
			public:
				DataConstIteratorA
				(
					DataA const & iData
				): Data(iData)
				{};

				DataConstIteratorA
				(
					DataConstIteratorA const & iDataConstIterator
				): Data(iDataConstIterator.Data), Cursor(iDataConstIterator.Cursor)
				{};

				virtual ~DataConstIteratorA() =0;

				virtual void First() =0;

				virtual void Next() =0;

				virtual DataType const & CurrentItem() const =0;

				virtual bool IsDone() const =0;

				virtual DataType const & operator[]
				(
					unsigned int iIndex
				) =0;

			protected:

				DataA const & Data;

				unsigned int Cursor;

			private:

		};


		/** This iterator should be used when the data is to be changed.
		 *
		 * @memo Changing data
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		class DataIteratorA
		{
			public:
				DataIteratorA
				(
					DataA & iData
				): Data(iData)
				{};

				DataIteratorA
				(
					DataIteratorA const & iDataIterator
				): Data(iDataIterator.Data), Cursor(iDataIterator.Cursor)
				{};

				virtual ~DataIteratorA() =0;

				virtual void First() =0;

				virtual void Next() =0;

				virtual DataType & CurrentItem() const =0;

				virtual bool IsDone() const =0;

				virtual DataType & operator[]
				(
					unsigned int iIndex
				) =0;

			protected:

				DataA & Data;

				unsigned int Cursor;

			private:

		};

		DataA()
		{};

		virtual ~DataA() =0;

		virtual unsigned int GetSize() const =0;

		virtual DataType & Get
		(
			unsigned int iIndex
		) const =0;

		/**
		 *
		 * These functions can be used to create iterators for traversing the data. Also provided are functions to copy existing iterators.
		 *
		 * @name Iterator creators
		 *
		 * @memo Creating iterators
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		//@{
		///
		virtual DataIteratorA* CreateDataIterator() =0;

		///
		virtual DataIteratorA* CreateDataIterator
		(
			DataIteratorA const & iDataIterator
		) =0;

		///
		virtual DataConstIteratorA* CreateDataConstIterator() const =0;

		///
		virtual DataConstIteratorA* CreateDataConstIterator
		(
			DataConstIteratorA const & iDataConstIterator
		) const =0;
		//@}

		friend ostream & operator<<
		(
			ostream & in,
			DataA const & iData
		)
		{
			iData.PrintToStream(in);
			return in;	
		};

	protected:
		
		virtual void PrintToStream
		(
			ostream & in
		) const =0;

	private:

}; // DataA

#endif

// Eof DataA.h
