/*
 * ReproductionA.h - A base class for a reproduction unit.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#ifndef ReproductionA_h
#define ReproductionA_h 1

#include "ConfigurationA.h"
#include "StatisticsA.h"
#include "Core/PoolA.h"
#include "Core/CoreFactoryA.h"
#include "Problem/ProblemFactoryA.h"
#include "Reproduction/ReproductionFactoryA.h"
#include "Reproduction/ProduceOperatorA.h"
#include "Reproduction/ChangeOperatorA.h"
#include "Reproduction/SelectParentsA.h"
#include "Reproduction/SelectSurvivorsA.h"

class ProblemA;


/** This is the reproduction unit within the library. It is called by the population to evolve to the next generation. It makes a number of supporting objects to aid all the work that as to be done. This work depends on what kind of operators and selection mechanism the evolutionary algorithm will have to contain. Most evolutionary algorithms make use of these four kinds op operators and mechanisms: \begin{itemize} \item[SelectParents] Selecting which genotypes will be used for reproducing purposes. \item[ProduceOperator] Producing new genotypes \item[ChangeOperator] Changing existing genotypes \item[SelectSurvivors] Selecting who will be in the next generation \end{itemize}
 *
 *
 * @memo The population
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class ReproductionA 
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. Its children should create the following objects: \begin{itemize} \item[]{SelectParents --- Selecting which genotypes will be used for reproducing purposes.} \item[]{ProduceOperator --- This operator takes a number of existing genotypes (parents) and creates a number of new ones (children, offspring). Examples of such operators are recomination- and crossover-operators.} \item[]{ChangeOperator --- This operator takes a number of existing genotypes and alters them, without creating new genotypes. Examples of such operators are mutation- and inversion-operators.} \item[]{SelectSurvivors --- When a population is changed and grown in size, often the population has to be brought back to the previous size. To do this a selection mechanism is needed to choose who stays and who will be killed.} \end{itemize}
		 *
		 * @memo Creating a new population
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @param iCoreFactory The factory that is used to spawn objects from classes of the Core package, within this class. It is passed on to these spawned objects as well.
		 * @param iProblemFactory The factory that is used to spawn objects from classes of the Problem package, within this class. It is passed on to these spawned objects as well.
		 * @param iReproductionFactory The factory that is used to spawn objects from classes of the Reproduction package, within this class. It is passed on to these spawned objects as well.
		 * @param iStopCondition The reproduction unit has the right to terminate the evolutionary algorithm, for instance when the maximum number of operations allowed is reached.
		 * @param iRandom Random numbers for the operators.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		ReproductionA
		(
			ConfigurationA const & iConfiguration,
			CoreFactoryA const & iCoreFactory,
			ProblemFactoryA const & iProblemFactory,
			ReproductionFactoryA const & iReproductionFactory,
			StopConditionA const & iStopCondition,
			ProblemA const & iProblem,
			RandomA const & iRandom
		);

		virtual ~ReproductionA() =0;

		/** This is the only function of the reproduction unit. When called it will turn the operators and selection mechanins loose on the provided pool.
		 *
		 * @memo Evolving a population
		 *
		 * @param iPool This is the pool of genotypes the reproduction unit will work on.
		 * @param iFitnessFunction Used to evaluate newly generated genotypes.
		 * @param iStatistics Here we can store some statistics, for instance the number of operations performed.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual void operator()
		(
			PoolA & iPool,
			FitnessFunctionA const & iFitnessFunction,
			StatisticsA & iStatistics
		) =0;

	protected:

		ConfigurationA const & Configuration;

		ProblemFactoryA const & ProblemFactory;
		CoreFactoryA const & CoreFactory;
		ReproductionFactoryA const & ReproductionFactory;
		StopConditionA const & StopCondition;
		ProblemA const & Problem;
		RandomA const & Random;

		ProduceOperatorA * ProduceOperator;
		ChangeOperatorA * ChangeOperator;
		SelectParentsA * SelectParents;
		SelectSurvivorsA * SelectSurvivors;

	private:

}; // ReproductionA

#endif

// Eof ReproductionA.h
