/*
 * SelectSurvivorsA.h - Abstract base class defining the interface of a survivors selection mechanism.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 25-Jun-1998
 *
 */

#ifndef SelectSurvivorsA_h
#define SelectSurvivorsA_h 1

#include "Core/PoolA.h"
#include "Core/PoolIteratorA.h"
#include "Common/ArrayC.h"

/** This selection mechanism takes a pool and a list containing the offspring. It then merges these two to form the pool for the next generation.
 *
 * @memo The survivors selection mechanism
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class SelectSurvivorsA
{
	public:

		/** This constructor should only be called by its children. It copies the reference of the parameters into protected members, they can then be used by its children to spawn objects and read configuration parameters. 
		 *
		 * @memo Creating a new survivors selection mechanism
		 *
		 * @param iConfiguration The configuration used to determine settings for the algorithm and to provide a configuration to spawned objects within this class.
		 * @return Nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */

		SelectSurvivorsA
		(
			ConfigurationA const & iConfiguration
		);

		virtual ~SelectSurvivorsA();


		/** \begin{description} \item[]{\tt ArrayC<GenotypeA*>* Children;} \item[]{\em #/##/# Produce a list of children and store them in the {\tt Children} list} \item[]{\tt SelectSurvivors(Pool, *Children);} \end{description} 
		 * Given a pool and a list of genotypes, the two are merged into a new pool which overwrites the old one.
		 *
		 * @memo Creating the pool for the next generation
		 *
		 * @param iPool This is the pool of the current generation.
		 * @param iArray This is a list of offspring that was created.
		 * @return iPool is the returned variable.
		 *
		 * @author J.I. van Hemert
		 * @version 0.1
		 *
		 */
		
		virtual void operator()
		(
			PoolA & iPool,
			ArrayC<GenotypeA*> const & iArray
		) const =0;
		
	protected:

		ConfigurationA const & Configuration;

	private:

}; // SelectSurvivorsA

#endif

// Eof SelectSurvivorsA.h
